/**********************************************************************
  PROJECT: Pixel and Line Demo
  Source: PXI_demo.cpp
  Chapter reference: 6
***********************************************************************
  Description:
    Demonstration program for drawing pixels, lines, and curves.
	All screen drawing operations take place during the WM_PAINT
	message intercept. This result is that the client area
    is maintained correctly by the code.
  Topics:
    1. Drawing in response to WM_PAINT 
    2. Use of SetPixelV() function to plot a curve
	3. Demonstration of line- and curve-drawing functions:
	   LineTo()
	   Polyline()
	   PolylineTo()
	   PolyPolyline()
	   Arc()
	   AngleArc()
	   PolyBezier()
	   PolyDraw()
***********************************************************************/

#include <windows.h>
#include "resource.h"
#include <math.h>


// Global variable for handle to program instance
HINSTANCE   pInstance;


// Function prototypes
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
BOOL AngleArc2 (HDC, INT, INT, DWORD, FLOAT, FLOAT);
BOOL PolyDraw95 (HDC, LPPOINT, LPBYTE, INT);
BOOL PolyDraw95A (HDC, LPPOINT, LPBYTE, INT);
VOID DrawDot (HDC, INT, INT);


//****************************
//         WinMain
//****************************
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static char szAppName[] = "Demo" ; // Application name
     HWND        hwnd ;
     MSG         msg ;

     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
     wndclass.lpszClassName = szAppName ; 
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

     // Store handle to instance
	 pInstance = hInstance;
     
	 // Registering the structure wmdclass
     RegisterClassEx (&wndclass) ;

     // CreateWindow()
     hwnd = CreateWindow (szAppName,
           "Pixel and Line Demo",         // window caption
            WS_OVERLAPPEDWINDOW,     // window style
            CW_USEDEFAULT,           // initial x position
            CW_USEDEFAULT,           // initial y position
            CW_USEDEFAULT,           // initial x size
            CW_USEDEFAULT,           // initial y size
            NULL,                    // parent window handle
            NULL,                    // window menu handle
            hInstance,               // program instance handle
            NULL) ;                  // creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

     // Message loop
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
}
//****************************
//    Windows Procedure
//****************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam,
LPARAM lParam)
{
     static int  cxChar, cxCaps, cyChar, cxClient, cyClient ;

     static HDC         hdc ;
     PAINTSTRUCT ps ;
     TEXTMETRIC  tm ;
	 static double pix2 = 2 * 3.14159;

	 // Variables for line drawing operations
	 static		HPEN       bluePen4, redPen2, blackPenDash;
	 static		HPEN       blackPenSol, blackPenDot;
	 
	 // Variable for pixel operations
	 COLORREF   pixColor;
	 int        x, y, i;


	 // Arrays of structures
	 POINT      pointsArray[18];     // array of points
	 DWORD		vertexArray[2];
	 BYTE       controlArray[18];

	// Drawing command selector
	static int	drawMode = 0;
	//                     0 = no menu command active
	//                         Active menu command:
	//                     1 = Set Pixel
	//					   2 = LineTo
	//                     3 = Polyline
	//					   4 = PolylineTo
	//                     5 = PolyPolyline
	//                     6 = Arc
	//                     7 = AngleArc
	//                     8 = PolyBezier
	//                     9 = PolyDraw

     switch (iMsg)
         {
     // Windows message processing
	      case WM_CREATE:
			 hdc = GetDC(hwnd);    // Obtain handle to private DC
              // Text metrics are obtained after font is selected 
              GetTextMetrics (hdc, &tm) ;
              cxChar = tm.tmAveCharWidth ;
              cxCaps = (tm.tmPitchAndFamily & 1 ? 3 : 2) * cxChar / 2 ;
              cyChar = tm.tmHeight + tm.tmExternalLeading ;
               // Create pens
			  blackPenDash = CreatePen(PS_DASH, 1, 0);
			  blackPenSol = CreatePen (PS_SOLID, 1, 0);
			  blackPenDot = CreatePen (PS_DOT, 1, 0);
			  bluePen4 = CreatePen(PS_SOLID, 4, RGB(0x00, 0x00, 0xff));
	    	  redPen2 = CreatePen(PS_SOLID, 2, RGB(0xff, 0x00, 0x00));
              return 0;

          case WM_PAINT :
               BeginPaint (hwnd, &ps) ;
		   	   switch(drawMode)
			   {
					case 1:
						pixColor = RGB(0xff, 0x0, 0x0); // Red
							for (i = 0; i < 1000; i++) {
							x = i * cxClient / 1000;
							y = (int) (cyClient / 2 *
							(1- sin (pix2 * i / 1000)));
							SetPixelV (hdc, x, y, pixColor);
						}
						break;

					case 2:
						// Create a solid blue pen, 4 pixels wide
						SelectObject(hdc, bluePen4);
						MoveToEx (hdc, 140, 140, NULL);
						LineTo (hdc, 300, 140);
						LineTo (hdc, 300, 300);
						LineTo (hdc, 140, 300);
						LineTo (hdc, 140, 140);
						break;

					case 3:
						// Create a solid red pen, 2 pixels wide
						SelectObject(hdc, redPen2);
						// Fill array of points
						pointsArray[0].x = 300; pointsArray[0].y = 160;
						pointsArray[1].x = 300; pointsArray[1].y = 300;
						pointsArray[2].x = 160; pointsArray[2].y = 300;
						pointsArray[3].x = 160; pointsArray[3].y = 160;
						MoveToEx (hdc, 160, 160, NULL);
						PolylineTo (hdc, pointsArray, 4);
						break;

					case 4:
						SelectObject(hdc, blackPenDash);
						// Fill array of points
						pointsArray[0].x = 160; pointsArray[0].y = 160;
						pointsArray[1].x = 300; pointsArray[1].y = 160;
						pointsArray[2].x = 300; pointsArray[2].y = 300;
						pointsArray[3].x = 160; pointsArray[3].y = 300;
						pointsArray[4].x = 160; pointsArray[4].y = 160;
						Polyline (hdc, pointsArray, 5);
						break;

					case 5:
						// Create a dashed black pen, 1 pixels wide
						SelectObject(hdc, redPen2);
						// Fill array of points for first polyline
						pointsArray[0].x = 160; pointsArray[0].y = 160;
						pointsArray[1].x = 300; pointsArray[1].y = 160;
						pointsArray[2].x = 300; pointsArray[2].y = 300;
						pointsArray[3].x = 160; pointsArray[3].y = 300;
						pointsArray[4].x = 160; pointsArray[4].y = 160;
						// Fill array of points for second polyline
						pointsArray[5].x = 160; pointsArray[5].y = 230;
						pointsArray[6].x = 230; pointsArray[6].y = 160;
						pointsArray[7].x = 300; pointsArray[7].y = 230;
						pointsArray[8].x = 230; pointsArray[8].y = 300;
						pointsArray[9].x = 160; pointsArray[9].y = 230;
						// Fill number of vertices in array
						vertexArray[0] = 5;
						vertexArray[1] = 5;
						PolyPolyline (hdc, pointsArray, vertexArray, 2);
						break;

					case 6:
						// Select the solid black pen
						// and draw bounding box
						SelectObject(hdc, blackPenSol);
						MoveToEx (hdc, 150, 150, NULL);
						LineTo (hdc, 350, 150);
						LineTo (hdc, 350, 250);
						LineTo (hdc, 150, 250);
						LineTo (hdc, 150, 150);
						// Select the dot black pen and draw
						// arc start/end lines
						SelectObject (hdc, blackPenDot);
						MoveToEx (hdc, 250, 200, NULL);
						LineTo (hdc, 250, 260);
						MoveToEx (hdc, 250, 200, NULL);
						LineTo (hdc, 200, 140);
						SelectObject(hdc, redPen2);
						// Draw the arc with the red pen
						Arc (hdc, 150, 150,    // Upper-left corner
							    350, 250,    // lower-right corner
								250, 260,    // start point
								200, 140);   // end point
						break;

					case 7:
						// Select the dot black pen and draw
						// coordinate axes
						SelectObject (hdc, blackPenDot);
						MoveToEx (hdc, 90, 200, NULL);
						LineTo (hdc, 310, 200);
						MoveToEx (hdc, 200, 310, NULL);
						LineTo (hdc, 200, 90);
						SelectObject (hdc, blackPenSol);
						// Draw a circle using the black pen
						Arc (hdc, 100, 100,    // Upper-left corner
					        300, 300,    // lower-right corner
							200, 100,    // start point
							200, 100);   // end point

						 SelectObject(hdc, redPen2);
						MoveToEx (hdc, 200, 200, NULL);
						AngleArc2 (hdc,
					        200, 200,
							100,
							15.0,
							45.0);
						break;

					case 8:
						// Draw dots at control points
						DrawDot(hdc, 200, 75);
						DrawDot(hdc, 280, 190);
						// Draw lines from nodes to control points
						SelectObject (hdc, blackPenDot);
						MoveToEx (hdc, 150, 150, NULL);
						LineTo (hdc, 200, 75);
						MoveToEx (hdc, 350, 150, NULL);
						LineTo (hdc, 280, 190);
						// Fill array of points for Bezier spline
						// Entries 0 and 3 are nodes
						// Entries 1 and 2 are control points
						pointsArray[0].x = 150; pointsArray[0].y = 150;
						pointsArray[1].x = 200; pointsArray[1].y = 75;
						pointsArray[2].x = 280; pointsArray[2].y = 190;
						pointsArray[3].x = 350; pointsArray[3].y = 150;
						SelectObject(hdc, redPen2);
						// Draw a Bezier spline
						PolyBezier (hdc, pointsArray, 4);
						break;

					case 9:
						// Move pen to start point 
						MoveToEx (hdc, 150, 50, NULL);
						// Fill array of points for three lines
						// offset:       purpose:
						//   0           end point of line 1
						//   1           start of line 2
						//   2           end of line 2
						//   3           start of line 3
						//   4           end of line 3
						pointsArray[0].x = 250; pointsArray[0].y = 50; // line
						pointsArray[1].x = 150; pointsArray[1].y = 70; // move
						pointsArray[2].x = 250; pointsArray[2].y = 70; // line
						pointsArray[3].x = 150; pointsArray[3].y = 90; // move
						pointsArray[4].x = 250; pointsArray[4].y = 90; // line
						// Move to start node of Bezier curve
						pointsArray[5].x = 150; pointsArray[5].y = 150;
						// Fill array of points for first Bezier spline
						pointsArray[6].x = 200; pointsArray[6].y = 75;
						pointsArray[7].x = 280; pointsArray[7].y = 190;
						pointsArray[8].x = 350; pointsArray[8].y = 150;
						// Fill array for closed figure
						pointsArray[9].x = 200; pointsArray[9].y = 200; // move
						pointsArray[10].x = 300; pointsArray[10].y = 200; // line
						pointsArray[11].x = 300; pointsArray[11].y = 300; // line
						pointsArray[12].x = 200; pointsArray[12].y = 300; // line
						// Fill array for second Bezier spline
						pointsArray[13].x = 300; pointsArray[13].y = 90; // move
						pointsArray[14].x = 350; pointsArray[14].y = 40;
						pointsArray[15].x = 350; pointsArray[15].y = 40;
						pointsArray[16].x = 400; pointsArray[16].y = 90;
						// Fill control array
						controlArray[0] = PT_LINETO;
						controlArray[1] = PT_MOVETO;
						controlArray[2] = PT_LINETO;
						controlArray[3] = PT_MOVETO;
						controlArray[4] = PT_LINETO;
						controlArray[5] = PT_MOVETO;
						controlArray[6] = PT_BEZIERTO;
						controlArray[7] = PT_BEZIERTO;
						controlArray[8] = PT_BEZIERTO;
						controlArray[9] = PT_MOVETO;
						controlArray[10] = PT_LINETO;
						controlArray[11] = PT_LINETO;
						controlArray[12] = PT_LINETO | PT_CLOSEFIGURE;
						controlArray[13] = PT_MOVETO;
						controlArray[14] = PT_BEZIERTO | PT_CLOSEFIGURE; 
						controlArray[15] = PT_BEZIERTO;
						controlArray[16] = PT_BEZIERTO;
						SelectObject(hdc, redPen2);
						// Draw lines and Bezier spline
						PolyDraw95A (hdc, pointsArray, controlArray, 17);
						break;

					default:
					break;

			   }
			   EndPaint(hwnd, &ps);

		   break;

          case WM_SIZE:
			  // Determine and store size of client area
			  cxClient = LOWORD(lParam);
			  cyClient = HIWORD(lParam);
			  return 0;

          //********************************
          //     menu command processing
          //********************************
		  case WM_COMMAND:
			  switch (LOWORD (wParam)) {

              //****************************
              //    pixel draw operation
			  //****************************
			  case ID_DRAWOP_PIXELDRAW:
				  drawMode = 1;   // Command to draw line
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

              //****************************
              //    LineTo() function
			  //****************************
			  case ID_DRAWOP_LINE_LINETO:
				  drawMode = 2;   // Command to draw line
				  InvalidateRect(hwnd, NULL, TRUE);
      			  break;

              //****************************
              //    PolyLineTo() function
			  //****************************
			  case ID_DRAWOP_LINES_POLYTO:
				  drawMode = 3;
				  InvalidateRect(hwnd, NULL, TRUE);
     			  break;
				  
              //****************************
              //    PolyLine() function
			  //****************************
			  case ID_DRAWOP_LINES_POLY:
				  drawMode = 4;
				  InvalidateRect(hwnd, NULL, TRUE);
     			  break;

              //****************************
              //  PolyPolyLine() function
			  //****************************				  
			  case ID_DRAWOP_LINES_POLYPOLY:
				  drawMode = 5;
				  InvalidateRect(hwnd, NULL, TRUE);
     			  break;

              //****************************
              //       Arc() function
			  //****************************
			  case ID_DRAWOP_LINES_ARC:
				  drawMode = 6;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

              //****************************
              //    AngleArc() function
			  //****************************				  
			  case ID_DRAWOP_LINES_ANGLE:
				  drawMode = 7;
				  InvalidateRect(hwnd, NULL, TRUE);
				  break;

              //****************************
              //    PolyBezier) function
			  //****************************
			  case ID_DRAWOP_LINES_POLYB:
				  drawMode = 8;
				  InvalidateRect(hwnd, NULL, TRUE);
     			  break;

			  //****************************
              //    PolyDraw() function
			  //****************************
			  case ID_DRAWOP_LINES_PDRAW:
				  drawMode = 9;
				  InvalidateRect(hwnd, NULL, TRUE);
      			  break;

			  case ID_DRAWOP_EXIT:
				  SendMessage(hwnd, WM_DESTROY, 0, 0);
				  return 0;
		  }
		  return 0;

          // End of program execution
          case WM_DESTROY :
			   // Delete all graphics objects
			   DeleteObject (bluePen4);
			   DeleteObject (blackPenDash);
			   DeleteObject (blackPenSol);
			   DeleteObject (blackPenDot);
	    	   DeleteObject (redPen2);

               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}

BOOL AngleArc2(HDC hdc, int X, int Y, DWORD dwRadius, 
               float fStartDegrees, float fSweepDegrees) {
 
  int iXStart, iYStart;  // End point of starting radial line
  int iXEnd, iYEnd;        // End point of ending radial line
  float fStartRadians;   // Start angle in radians
  float fEndRadians;     // End angle in radians
  BOOL bResult;            // Function result
  float fTwoPi = 2.0f * 3.141592f;

  /* Get the starting and ending angle in radians */
  if (fSweepDegrees > 0.0f) {
    fStartRadians = ((fStartDegrees / 360.0f) * fTwoPi);
    fEndRadians = (((fStartDegrees + fSweepDegrees) / 360.0f) *
          fTwoPi);
  } else {
    fStartRadians = (((fStartDegrees + fSweepDegrees)  / 360.0f) *
          fTwoPi);
    fEndRadians =  ((fStartDegrees / 360.0f) * fTwoPi);
  }

  /* Calculate a point on the starting radial line via */
  /* polar -> cartesian conversion */
  iXStart = X + (int)((float)dwRadius * (float)cos(fStartRadians));
  iYStart = Y - (int)((float)dwRadius * (float)sin(fStartRadians));

  /* Calculate a point on the ending radial line via */
  /* polar -> cartesian conversion */
  iXEnd = X + (int)((float)dwRadius * (float)cos(fEndRadians));
  iYEnd = Y - (int)((float)dwRadius * (float)sin(fEndRadians));

  /* Draw a line to the starting point */
  LineTo(hdc, iXStart, iYStart);

  /* Draw the arc */
  bResult = Arc(hdc, X - dwRadius, Y - dwRadius,
                     X + dwRadius, Y + dwRadius,
                     iXStart, iYStart,
                     iXEnd, iYEnd);

  /* Move to the ending point - Arc() wont do this and ArcTo() */
  /* wont work on Win32s or Win16 */
  MoveToEx(hdc, iXEnd, iYEnd, NULL);

  return bResult;
} 

//*****************************
// Win95 version of PolyDraw()
//*****************************
// Not used in this program
BOOL PolyDraw95(HDC  hdc,                // handle of a device context
                CONST LPPOINT lppt,      // array of points
                CONST LPBYTE lpbTypes,   // line and curve identifiers
                int  cCount)             // count of points
{ 
  int i;

  for (i=0; i<cCount; i++)
    switch (lpbTypes[i]) {
      case PT_MOVETO :
         MoveToEx(hdc, lppt[i].x, lppt[i].y, NULL);
         break;

      case PT_LINETO | PT_CLOSEFIGURE:
      case PT_LINETO :
         LineTo(hdc, lppt[i].x, lppt[i].y);
         break;

      case PT_BEZIERTO | PT_CLOSEFIGURE:
      case PT_BEZIERTO :
        PolyBezierTo(hdc, &lppt[i], 3);
       i+=2;
         break;
    }

   return TRUE;
} 


//*****************************
// Win95 version of PolyDraw()
//        improved!
//*****************************
BOOL PolyDraw95A (HDC  hdc,                // handle of a device context
                CONST LPPOINT lppt,      // array of points
                CONST LPBYTE lpbTypes,   // line and curve identifiers
                int  cCount)             // count of points
{ 
  int i;
  static long lastPenx, lastPeny;   // Storage for last pen position
  POINT       currentPoints[1];


  // Store initial position of drawing pen
     GetCurrentPositionEx (hdc, currentPoints);
	 lastPenx = currentPoints[0].x;
	 lastPeny = currentPoints[0].y;

  for (i=0; i<cCount; i++)
    switch (lpbTypes[i]) {

      case PT_MOVETO :
         MoveToEx(hdc, lppt[i].x, lppt[i].y, NULL);
	 	 lastPenx = lppt[i].x;
		 lastPeny = lppt[i].y;
         break;

      case PT_LINETO | PT_CLOSEFIGURE:
         LineTo(hdc, lppt[i].x, lppt[i].y);
		 LineTo(hdc, lastPenx, lastPeny);
         break;

      case PT_LINETO :
         LineTo(hdc, lppt[i].x, lppt[i].y);
         break;

      case PT_BEZIERTO | PT_CLOSEFIGURE:
        // Store start points of Bezier for closing
		GetCurrentPositionEx (hdc, currentPoints);
		lastPenx = currentPoints[0].x;
		lastPeny = currentPoints[0].y;
		PolyBezierTo(hdc, &lppt[i], 3);
		i+=2;
		LineTo(hdc, lastPenx, lastPeny);
		break;

      case PT_BEZIERTO :
		 // Draw Bezier
         PolyBezierTo(hdc, &lppt[i], 3);
         i+=2;
		 break;

    }

   return TRUE;
} 

void DrawDot (HDC hdc, int x, int y) {

  COLORREF    blackPix;

  // Draw a five-pixel dot
  blackPix = RGB(0x0, 0x0, 0x0); // Black
  SetPixelV (hdc, x, y, blackPix);
  SetPixelV (hdc, x-1, y, blackPix);
  SetPixelV (hdc, x+1, y, blackPix);
  SetPixelV (hdc, x, y-1, blackPix);
  SetPixelV (hdc, x, y+1, blackPix);
  return;

}


